/*
 * Copyright (C) 2009-2019 Lightbend Inc. <https://www.lightbend.com>
 */

package akka.persistence.query.journal.leveldb

import akka.stream.actor.ActorPublisher
import com.github.ghik.silencer.silent

/**
 * INTERNAL API
 */
@silent // FIXME Re-write as part of https://github.com/akka/akka/issues/26187
private[akka] trait DeliveryBuffer[T] { _: ActorPublisher[T] =>

  var buf = Vector.empty[T]

  def deliverBuf(): Unit =
    if (buf.nonEmpty && totalDemand > 0) {
      if (buf.size == 1) {
        // optimize for this common case
        onNext(buf.head)
        buf = Vector.empty
      } else if (totalDemand <= Int.MaxValue) {
        val (use, keep) = buf.splitAt(totalDemand.toInt)
        buf = keep
        use.foreach(onNext)
      } else {
        buf.foreach(onNext)
        buf = Vector.empty
      }
    }

}
